/**
 * Created by Sam R.
 * on 17-12-2011
 * at 09:39
 */
package com.cirrial.ld22 {
import com.cirrial.ld22.widgets.Barrel;
import com.cirrial.ld22.widgets.Door;
import com.cirrial.ld22.widgets.Elevator;
import com.cirrial.ld22.widgets.Generator;
import com.cirrial.ld22.widgets.Laser;
import com.cirrial.ld22.widgets.LaserEmitter;
import com.cirrial.ld22.widgets.Spark;
import com.cirrial.ld22.widgets.SparkEmitter;
import com.cirrial.ld22.widgets.Terminal;
import com.cirrial.ld22.widgets.Powerable;

import org.flixel.FlxCamera;

import org.flixel.FlxG;

import org.flixel.FlxGroup;
import org.flixel.FlxObject;
import org.flixel.FlxState;
import org.flixel.FlxText;
import org.flixel.FlxTilemap;

public class PlayState extends FlxState {

    public static var player:Player;
    private static var widgetsGroup:FlxGroup;
    private static var backgroundWidgetsGroup:FlxGroup;
    private static var allPowerablesGroup:FlxGroup;
    private static var sparksGroup:FlxGroup;
    private static var lasersGroup:FlxGroup;

    private static var map:FlxTilemap;

    private var statusText:FlxText;
    private var terminalText:FlxText;

    private var generator:Generator;
    private var gameEndingElevator:Elevator;
    public static var transition:int = 0; // -1 means death, 1 means victory

    private static var actionKeyTimer:Number = 0; // prevents rapid retriggering of things

    override public function create():void {
        // initialise
        widgetsGroup = new FlxGroup();
        backgroundWidgetsGroup = new FlxGroup();
        sparksGroup = new FlxGroup();
        allPowerablesGroup = new FlxGroup();
        lasersGroup = new FlxGroup();

        var level:BaseLevel = new Level_Facility(true, onObjectAddedCallback);
        FlxG.camera.follow(player,  FlxCamera.STYLE_PLATFORMER);
        FlxG.camera.setBounds(0, 0, BaseLevel.boundsMaxX, BaseLevel.boundsMaxY, true);
        FlxG.bgColor = level.bgColor;

        player.getPowerables(allPowerablesGroup);
        generator.setPowerables(allPowerablesGroup);

        add(sparksGroup);
        add(lasersGroup);

        statusText = new FlxText(0, 10, 320);
        statusText.scrollFactor.x = 0.0;
        statusText.scrollFactor.y = 0.0;
        statusText.setFormat(null, 24, 0xFFA0C0FF, "center", 0xFF000000);
        add(statusText);

        terminalText = new FlxText(10, 180, 300);
        terminalText.scrollFactor.x = 0.0;
        terminalText.scrollFactor.y = 0.0;
        terminalText.setFormat(null, 8, 0xFFA0C0FF, "left", 0xFF000000);
        add(terminalText);

        FlxG.flash(0xFF000000, 2.0);
        FlxG.playMusic(SoundData.MusAmbience);

        terminalText.text = "Arrow keys move. X jumps, C interacts.";
        transition = 0;
    }

    override public function update():void {
        // update

        // collisions
        FlxG.collide(player, map);
        FlxG.collide(sparksGroup, map, function(spark:Spark, map:FlxTilemap):void { spark.kill() } );
        FlxG.collide(player, widgetsGroup);

        // game logic
        if(!player.isDying) {
            FlxG.collide(player, sparksGroup,
                    function(player:Player, spark:Spark):void {
                        inflictDamage(player, spark, 2);
                        spark.kill();
                    } );
            FlxG.collide(player, lasersGroup,
                    function(player:Player, laser:Laser):void {
                        inflictDamage(player, laser, 10 * Main.EXPECTED_TIMESTEP);
                    });
            FlxG.overlap(player, backgroundWidgetsGroup, onWidgetOverlap);

            // player is dying slowly :c
            player.adjustEnergy(Main.DRAIN_RATE * Main.EXPECTED_TIMESTEP * -1);

            // check to see if the player is trying to open a door

            statusText.text = "" + (int)(player.energy);
        } else {
            if(transition == 0) {
                transition = -1;
                FlxG.fade(0xFF000000, 1.0, function():void {
                    FlxG.resetState();
                });
            }
        }

        if(actionKeyTimer>0) actionKeyTimer -= Main.EXPECTED_TIMESTEP;
        super.update();
    }

    private function inflictDamage(player:Player, source:FlxObject, damage:Number):void {
        var dx:Number = source.x - player.x;
        var dir:Number = -1 * dx/Math.abs(dx);
        player.hurt(damage);
        player.velocity.x += dir * 300;
        player.velocity.y += -100;
        Main.playSound(SoundData.SndHit, source);
    }

    private function onWidgetOverlap(player:Player, obj:FlxObject):void {
        if(FlxG.keys.justPressed(Main.KEY_ACTION) && actionKeyTimer<=0) {
            if(obj is Elevator) {
                var elevator:Elevator = obj as Elevator;
                if(elevator != gameEndingElevator) {
                    elevator.transport(player);
                } else {
                    if(transition == 0) {
                        transition = 1;
                        FlxG.fade(0xFFFFFFFF, 1.0, function():void {
                            FlxG.switchState(new CutsceneState());
                        })
                    }
                }
            } else if(obj is Terminal) {
                var id:uint = (obj as Terminal).getTextId();
                if(id >= 0 && id < TerminalStrings.TERMINAL_STRINGS.length) {
                    terminalText.text = TerminalStrings.TERMINAL_STRINGS[id];
                }
            }
            actionKeyTimer = 0.1;
        }
    }


    // I am so sorry for the following chunk of code
    private function onObjectAddedCallback(obj:Object, group:FlxGroup, lvl:BaseLevel, scrollX:Number, scrollY:Number, properties:Array):Object {
        var s:String;
        if(obj is Player) {
            player = obj as Player;
        } else if(obj is FlxTilemap) {
            map = obj as FlxTilemap;
        } else if(obj is Door) {
            var door:Door = obj as Door;
            allPowerablesGroup.add(door);
            widgetsGroup.add(door);
        } else if(obj is Terminal) {
            var terminal:Terminal = obj as Terminal;
            var id:uint = 0;
            for (s in properties) {
                switch(properties[s].name) {
                    case "TerminalID":
                        id = properties[s].value;
                        break;
                }
            }
            terminal.assignId(id);
            backgroundWidgetsGroup.add(terminal);
        } else if(obj is Elevator) {
            var elevator:Elevator = obj as Elevator;
            for (s in properties) {
                switch(properties[s].name) {
                    case "EndsGame":
                        if(properties[s].value) {
                            gameEndingElevator = elevator;
                        }
                    break;
                }
            }
            allPowerablesGroup.add(elevator);
            backgroundWidgetsGroup.add(elevator);
        } else if(obj is Barrel) {
            var barrel:Barrel = obj as Barrel;
            allPowerablesGroup.add(barrel);
            backgroundWidgetsGroup.add(barrel);
        } else if(obj is Generator) {
            var gen:Generator = obj as Generator;
            generator = gen;
            allPowerablesGroup.add(gen);
            backgroundWidgetsGroup.add(gen);
        } else if(obj is SparkEmitter) {
            var sparkEmitter:SparkEmitter = obj as SparkEmitter;
            sparkEmitter.giveGroup(sparksGroup);
            backgroundWidgetsGroup.add(sparkEmitter);
        } else if(obj is LaserEmitter) {
            var laserEmitter:LaserEmitter = obj as LaserEmitter;
            allPowerablesGroup.add(laserEmitter);
            backgroundWidgetsGroup.add(laserEmitter);
        } else if(obj is ObjectLink) {
            var link:ObjectLink = obj as ObjectLink;
            if(link.toObject is Elevator) {
                var elevatorOne:Elevator = link.toObject as Elevator;
                var elevatorTwo:Elevator = link.fromObject as Elevator;
                elevatorOne.setTarget(elevatorTwo);
                elevatorTwo.setTarget(elevatorOne);
            } else if(link.toObject is LaserEmitter) {
                var emitterOne:LaserEmitter = link.toObject as LaserEmitter;
                var emitterTwo:LaserEmitter = link.fromObject as LaserEmitter;
                if(emitterOne && emitterTwo) {
                    var laser:Laser = new Laser(emitterOne, emitterTwo);
                    lasersGroup.add(laser);
                }
            }
        }
        return obj;
    }
}
}
